from zope.interface import implements
from zope.component import getUtilitiesFor
from zope.component import getUtility
from Solgema.FlowView.interfaces import IFlowViewDisplayType
from zope.component import getMultiAdapter
from settings import FlowViewSettings
from interfaces import IFlowViewSettings
from Solgema.FlowView.config import DISPLAY_NAME_VIEW_PREFIX
from vocabularies import FlowViewDisplayTypeVocabulary


def getDisplayAdapter(context, request, display_type=None):
    if display_type is None:
        display_type = FlowViewSettings(context).display_type

    possible_types = FlowViewDisplayTypeVocabulary(context)
    if display_type not in possible_types.by_value.keys():
        display_type = IFlowViewSettings['display_type'].default

    return getMultiAdapter(
        (context, request),
        name=DISPLAY_NAME_VIEW_PREFIX + display_type
    )


def getDisplayType(name):
    return getUtility(IFlowViewDisplayType, name=DISPLAY_NAME_VIEW_PREFIX + name)


def getAllDisplayTypes():
    utils = list(getUtilitiesFor(IFlowViewDisplayType))
    utils = sorted(utils, key=lambda x: x[1].name)
    return [utility for name, utility in utils]


def createSettingsFactory(schema):
    class Settings(FlowViewSettings):
        implements(schema)

        def __init__(self, context, interfaces=[schema]):
            super(Settings, self).__init__(context, interfaces)

    return Settings



from zope.interface import Interface
        
class ISFWUtility(Interface):
    """
    utility methods for flowview
    """
    
    def should_include(filename):
        """
        This method can be called from an include expression for css/javascript
        to check if a file should be included for the flowview.
        """
        
    def enabled():
        """
        Lets you know if the flowview is enabled
        """



from zope.interface import implements
from persistent.dict import PersistentDict
try:
    #For Zope 2.10.4
    from zope.annotation.interfaces import IAnnotations
except ImportError:
    #For Zope 2.9
    from zope.app.annotation.interfaces import IAnnotations

from interfaces import IFlowViewSettings

class FlowViewSettings(object):
    """
    Just uses Annotation storage to save and retrieve the data...
    """
    implements(IFlowViewSettings)
    
    defaults = {}
    
    def __init__(self, context, interfaces=[IFlowViewSettings]):

        self.context = context
        
        self._interfaces = interfaces
        if type(self._interfaces) not in (list, tuple):
            self._interfaces = [self._interfaces]
        self._interfaces = list(self._interfaces)
        if IFlowViewSettings not in self._interfaces:
            self._interfaces.append(IFlowViewSettings)
        
        annotations = IAnnotations(context)

        self._metadata = annotations.get('Solgema.FlowView', None)
        if self._metadata is None:
            self._metadata = PersistentDict()
            annotations['Solgema.FlowView'] = self._metadata
            
    def __setattr__(self, name, value):
        if name in ('context', '_metadata', '_interfaces', 'defaults'):
            self.__dict__[name] = value
        else:
            self._metadata[name] = value
            
    def __getattr__(self, name):
        default = None
        
        if self.defaults.has_key(name):
            default = self.defaults[name]
        
        for iface in self._interfaces:
            if name in iface.names():
                default = iface[name].default
                        
        return self._metadata.get(name, default)




from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import getFSVersionTuple
from zope.component import adapter, getUtility, getAdapters, getMultiAdapter, getSiteManager
from Products.GenericSetup.interfaces import IProfileImportedEvent
import logging
LOG = logging.getLogger('Solgema.FlowView')

def installSolgemaFlowView(context):
    if context.readDataFile('solgemaflowview_various.txt') is None:
        return
    site = context.getSite()
    
    catalog = getToolByName(site, 'portal_catalog')
    folderish = catalog.searchResults(portal_type=['Folder', 'FolderRoot', 'MemberFolder', 'Topic'])
    for citem in folderish:
        item = citem.getObject()
        if not hasattr(item, 'getLayout'):
            LOG.info('Solgema Update: item %s has no getLayout attribute.'%(str(item)))
            continue
        if item.getLayout() == 'folder_full_view_tabs':
            item._updateProperty('layout', 'flowview')
    LOG.info('Solgema Update: old views replaced')

    setup = getToolByName(site, 'portal_setup')
    ttool = getToolByName(site, 'portal_types')
    types = [ttool.Folder,]
    if hasattr(ttool, 'Topic'):
        types.append(ttool.Topic)
    if hasattr(ttool, 'Collection'):
        types.append(ttool.Collection)
    for ttype in types:
        methods = ttype.view_methods
        if 'folder_full_view_tabs' in methods:
            ttype.manage_changeProperties(view_methods=tuple([a for a in methods if a !='folder_full_view_tabs']))

    if getFSVersionTuple()[0] == 4:
        setup.runAllImportStepsFromProfile('profile-Solgema.FlowView:plone4')
        jstool = getToolByName(site, 'portal_javascripts')
        jstool.cookResources()
        csstool = getToolByName(site, 'portal_css')
        csstool.cookResources()
    else:
        setup.runAllImportStepsFromProfile('profile-Solgema.FlowView:plone5')
        
def uninstallSolgemaFlowView(context):
    if context.readDataFile('solgemaflowview_uninstall.txt') is None:
        return
    site = context.getSite()

@adapter(IProfileImportedEvent)
def handleProfileImportedEvent(event):
    context = event.tool
    portal_quickinstaller = getToolByName(context, 'portal_quickinstaller')
    if portal_quickinstaller.isProductInstalled('Solgema.FlowView') and 'to500' in event.profile_id and event.full_import:
        portal_setup = getToolByName(context, 'portal_setup')
        portal_setup.runAllImportStepsFromProfile('profile-Solgema.FlowView:uninstall4')
        portal_setup.runAllImportStepsFromProfile('profile-Solgema.FlowView:plone5')




from zope.interface import implements
from plone.memoize.view import memoize

from Acquisition import aq_inner
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView

from interfaces import ISFWUtility
from Solgema.FlowView.settings import FlowViewSettings

class SFWUtility(BrowserView):
    """Information about the state of the portal
    """
    implements(ISFWUtility)

    @memoize
    def should_include(self, display_type):
        context = aq_inner(self.context)
        try:
            return self.enabled() and FlowViewSettings(context).display_type == display_type
        except TypeError:
            return False
            
    @memoize
    def enabled(self):
        utils = getToolByName(self.context, 'plone_utils')
        try:
            return utils.browserDefault(self.context)[1][0] == "flowview"
        except:
            return False
        



from Solgema.FlowView.config import _
from Products.CMFPlone import PloneMessageFactory as _pmf
from zope.interface import Interface
from plone.z3cform.fieldsets import group as plonegroup
from z3c.form import form, field as z3field, group, button
from plone.app.z3cform.layout import wrap_form
from Solgema.FlowView.interfaces import IFlowViewSettings
from Solgema.FlowView.utils import getDisplayType
from Solgema.FlowView.utils import getAllDisplayTypes
from Solgema.FlowView.settings import FlowViewSettings
import zope.i18n


class INothing(Interface):
    pass

class FlowViewSettingsForm(group.GroupForm, form.EditForm):
    """
    The page that holds all the flowview settings
    """

    fields = z3field.Fields()
    contentFields = ['content_layout',
                     'display_content_title',
                     'display_content_image',
                     'display_content_description',
                     'display_content_text',
                     'display_more_link',
                     'thumb_size',
                     'thumb_position',
                     'thumb_side']
    navigationFields = ['navigator', 'tab_content', 'tab_position', 'use_backnext']
    
    label = _(u'heading_flowview_settings_form', default=u'Flow View')
    
    description = _(u'description_flowview_settings_form',
        default=u'Configure the parameters for this Flow View.')

    successMessage = _(u'successMessage_flowview_settings_form',
        default=u'Flow View Settings Saved.')
    noChangesMessage = _(u'noChangesMessage_flowview_settings_form',
        default=u'There are no changes in the Flow View settings.')

    def update(self):
        fields = z3field.Fields(IFlowViewSettings)
#        fields['invocation_code'].field.default += u"activateFlowView($('#flow_%s'));" % (self.context.id, self.context.id)
#        fields['invocation_code'].field.description += u"activateFlowView($('#flow_%s'));" % (self.context.id)
        fields['invocation_code'].field.description = _(u'You can add your custom javascript invocation code here. Leave blank to use default. Default code is: ${default_code}',
            mapping={'default_code':u"activateFlowView($('#flow_%s'));" % (self.context.id)})
        fieldsid = [field.field.__name__ for field in fields.values()]
        main = plonegroup.GroupFactory(_pmf(u"label_choose_template"), fields.select(*[a for a in fieldsid if a not in self.contentFields+self.navigationFields]))
        content = plonegroup.GroupFactory(_pmf(u"label_schema_default"), fields.select(*[a for a in fieldsid if a in self.contentFields]))
        navigation = plonegroup.GroupFactory(_pmf(u"Navigation"), fields.select(*[a for a in fieldsid if a in self.navigationFields]))
        self.groups = [main, content, navigation]
        super(FlowViewSettingsForm, self).update()
        
    def set_status_message(self, settings, has_changes):
        msg = has_changes and self.successMessage or self.noChangesMessage
        msg = zope.i18n.translate(msg)
        self.status = msg

    @button.buttonAndHandler(_('Apply'), name='apply')
    def handleApply(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        changes = self.applyChanges(data)
        settings = FlowViewSettings(self.context)

        has_changes = False
        if changes:
#            settings = FlowViewSettings(self.context)
#            settings.last_cooked_time_in_seconds = 0
            has_changes = True

        self.set_status_message(settings, has_changes)

FlowViewSettingsView = wrap_form(FlowViewSettingsForm)



import logging
from zope import schema
from zope.interface import Interface
from Products.Archetypes.config import REFERENCE_CATALOG
from Products.Archetypes.interfaces.referenceable import IReferenceable
from Products.CMFCore.utils import getToolByName
from plone.app.uuid.utils import uuidToObject
from zope.annotation.interfaces import IAnnotations
from plone.uuid.interfaces import IUUID
from zope.component import getUtility
try:
    from plone.app.contenttypes.migration.migration import ICustomMigrator
    from plone.app.contenttypes.migration.field_migrators import migrate_filefield, migrate_simplefield
    from plone.app.contenttypes.migration.utils import link_items
    from z3c.relationfield.relation import RelationValue
except:
    class ICustomMigrator(Interface):
        """""" 
try:
    from zope.intid.interfaces import IIntIds
except ImportError:
    from zope.app.intid.interfaces import IIntIds
from Products.ATContentTypes.interfaces.interfaces import IATContentType
from zope.component import adapter
from Products.Five.utilities import marker
from zope.component import getAdapters
from zope.component import getMultiAdapter
from zope.component.hooks import getSite
from zope.interface import implementer
from zope.schema.interfaces import IBool
from zope.component import getAllUtilitiesRegisteredFor
from plone.dexterity.interfaces import IDexterityFTI, IDexterityContent
from Solgema.FlowView.interfaces import IFlowViewSettings, IFlowViewMarker
_logger = logging.getLogger(__name__)

@implementer(ICustomMigrator)
@adapter(IFlowViewMarker)
class FlowViewMigator(object):

    def __init__(self, context):
        self.context = context
    
    def migrate(self, old, new):
        marker.mark(new, IFlowViewMarker)
            
        for name, field in schema.getFieldsInOrder(IFlowViewSettings):
            migrate_simplefield(old, IFlowViewSettings(new), name, name)

        _logger.info(
            "Migrating FlowView Settings for %s" % new.absolute_url())



from plone.theme.interfaces import IDefaultPloneLayer
from zope.interface import Interface, Attribute
from zope import schema
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm
from Solgema.FlowView.config import _

class ISolgemaFlowViewLayer(IDefaultPloneLayer):
    """Solgema FlowView layer"""

class IFlowViewMarker(Interface):
    """
    marker interface for content types that implement
    the flowview
    """

class IFlowViewDisplayType(Interface):
    name = Attribute("name of display type")
    description = Attribute("description of type")
    schema = Attribute("Options for this display type")
    userWarning = Attribute("A warning to be displayed to to "
                            "the user if they use this type.")
    width = Attribute("The width of the flow view")
    height = Attribute("The height of the flow view")
    start_image_index = Attribute("What image the flow view should "
                                  "start playing at.")

    def content():
        """
        the content of the display yet
        """

    def javascript():
        """
        content to be included in javascript area of template
        """

    def css():
        """
        content to be included in css area of template
        """
    
class IFlowViewSettings(Interface):

    showTitle = schema.Bool(
        title=_(u"label_showTitle", default=u"Show Title?"),
        default=False,
        required=True
    )
    
    showDescription = schema.Bool(
        title=_(u"label_showDescription", default=u"Show Description?"),
        default=True,
        required=True
    )

    event = schema.Choice(
        title=_(u"label_event", default=u"Event"),
        description=_(u"description_event",
            default=u"Specifies the event when a tab is opened."
        ),
        default='click',
        vocabulary=SimpleVocabulary([
            SimpleTerm('click', 'click', _(u"label_click", default=u"click")),
            SimpleTerm('mouseover', 'mouseover', _(u"label_mouseover", default=u"mouseover")),
            SimpleTerm('dblclick', 'dblclick', _(u"label_dblclick", default=u"dblclick")),
        ]),
        required=True
    )

    effect = schema.Choice(
        title=_(u"label_effect", default=u"Effect"),
        description=_(u"description_effect",
            default=u"The effect to be used when a tab is clicked."
        ),
        default='default',
        vocabulary=SimpleVocabulary([
            SimpleTerm('default', 'default', _(u"label_default", default=u"default")),
            SimpleTerm('fade', 'fade', _(u"label_fade", default=u"fade")),
            SimpleTerm('ajax', 'ajax', _(u"label_ajax", default=u"ajax")),
            SimpleTerm('slide', 'slide', _(u"label_slide", default=u"slide")),
            SimpleTerm('horizontal', 'horizontal', _(u"label_horizontal", default=u"horizontal")),
            SimpleTerm('swing', 'swing', _(u"label_swing", default=u"swing")),
            SimpleTerm('linear', 'linear', _(u"label_linear", default=u"linear")),
            SimpleTerm('custom', 'custom', _(u"label_custom_effect", default=u"Custom Effect")),
        ]),
        required=True
    )

    custom_effect = schema.Text(
        title=_(u"label_custom_effect", default=u"Custom Effect"),
        description=_(u"help_custom_effect", default=u'You can add your custom javascript effect code here. Only works if "custom" selected as effect. Name of you effect must be "custom"'),
        required=False
    )
    
    fadeInSpeed = schema.Int(
        title=_(u"label_fadeInSpeed", default=u"fadeInSpeed"),
        description=_(u"description_fadeInSpeed",
            default=u"Only available when used together with the \"fade\" effect."
                    u"This property defines how fast (in milliseconds) the opened pane reveals its content."
        ),
        default=200,
        required=True
    )

    fadeOutSpeed = schema.Int(
        title=_(u"label_fadeOutSpeed", default=u"fadeOutSpeed"),
        description=_(u"description_fadeOutSpeed",
            default=u"Only available when used together with the \"fade\" effect."
                    u"This property defines how fast (in milliseconds) the opened pane hides its content."
        ),
        default=0,
        required=True
    )
    
    timed = schema.Bool(
        title=_(u"label_timed", default=u"Timed?"),
        description=_(u"description_timed",
            default=u"Should this Flow View automatically "
                    u"change images for the user?"
        ),
        default=False,
        required=True
    )

    interval = schema.Int(
        title=_(u"label_interval", default=u"Interval"),
        description=_(u"description_interval",
            default=u"If slide show is timed, the interval sets "
                    u"how long before the next image is shown in miliseconds."
        ),
        default=3000,
        required=True
    )

    speed = schema.Int(
        title=_(u"label_speed", default=u"speed"),
        description=_(u"description_speed",
            default=u"The time (in milliseconds) of the scrolling animation."
        ),
        default=400,
        required=True
    )

    height = schema.Int(
        title=_(u"label_height", default=u"Height"),
        description=_(u"description_height",
            default=u"The height (in px) of the flowview content. If empty, the tallest item's height will be used."
        ),
        required=False
    )

    initialIndex = schema.Int(
        title=_(u"label_initialIndex", default=u"Initial Index"),
        description=_(u"description_initialIndex",
            default=u"Specifies the tab that is initially opened when the page loads."
        ),
        default=0,
        required=True
    )
    
    autoplay = schema.Bool(
        title=_(u"label_autoplay", default=u"Auto play?"),
        description=_(u"description_autoplay",
            default=u"If this property is set to true then the tabs will automatically advance to the next tab implementing an automatically \"playable\" slideshow."
        ),
        default=False,
        required=True
    )
    
    autopause = schema.Bool(
        title=_(u"label_autopause", default=u"Auto pause?"),
        description=_(u"description_autopause",
            default=u"If this property is set to true then the tabs will automatically advance to the next tab implementing an automatically \"playable\" slideshow."
        ),
        default=True,
        required=True
    )
    
    vertical = schema.Bool(
        title=_(u"label_vertical", default=u"Vertical?"),
        description=_(u"description_vertical",
            default=u"If empty, the scrollable will guess by itself. "
        ),
        required=False
    )
    
    flowPadding = schema.Int(
        title=_(u"label_flowPadding", default=u"Padding in px"),
        default=15,
        required=False
    )
    
    showBorder = schema.Bool(
        title=_(u"label_showBorder", default=u"Show border"),
        default=True
    )
    
    showFirstOnly = schema.Bool(
        title=_(u"label_showFirstOnly", default=u"Show first pane only"),
        description=_(u"description_showFirstOnly",
            default=u"If your flowview is not used as content but only as \"costmetics\" with images, showing only first element avoid having the page to break on load or if javascipt is not activated."
        ),
        
        default=True
    )

# CONTENT FIELDS
    
    invocation_code = schema.Text(
        title=_(u"label_invocation_code", default=u"Custom Invocation Code"),
        description=_(u'You can add your custom javascript invocation code here. Leave blank to use default. Default code is: ${default_code}'),
        required=False
    )
    
    content_layout = schema.Choice(
        title=_(u"label_content_layout", default=u"Content Layout"),
        description=_(u"description_content_layout",
        ),
        default='content',
        vocabulary='Solgema.FlowView.FlowViews',
        required=True
    )
    
    thumb_size = schema.Choice(
        title=_(u"label_thumb_size", default=u"Thumbnail image size"),
        description=_(u"description_thumb_size",
            default=u"The size of thumbnail images. "
        ),
        default='tile',
        vocabulary=SimpleVocabulary([
            SimpleTerm('tile', 'tile', _(u"label_tile", default=u"tile")),
            SimpleTerm('thumb', 'thumb', _(u"label_thumb", default=u"thumb")),
            SimpleTerm('mini', 'mini', _(u"label_mini", default=u"mini")),
            SimpleTerm('preview', 'preview', _(u"label_preview", default=u"preview")),
        ]),
        required=True
    )
    
    thumb_position = schema.Choice(
        title=_(u"label_thumb_position", default=u"Thumbnail image position"),
        description=_(u"description_thumb_position",
            default=u"The position of thumbnail images. "
        ),
        default='top',
        vocabulary=SimpleVocabulary([
            SimpleTerm('top', 'top', _(u"label_top", default=u"top")),
            SimpleTerm('undertitle', 'undertitle', _(u"label_undertitle", default=u"undertitle")),
            SimpleTerm('bottom', 'bottom', _(u"label_bottom", default=u"bottom")),
        ]),
        required=True
    )
    
    thumb_side = schema.Choice(
        title=_(u"label_thumb_side", default=u"Thumbnail image side"),
        description=_(u"description_thumb_side",
            default=u"The side of thumbnail images. "
        ),
        vocabulary=SimpleVocabulary([
            SimpleTerm('left', 'left', _(u"label_left", default=u"Left")),
            SimpleTerm('right', 'right', _(u"label_right", default=u"Right")),
        ]),
        required=False
    )
    
    display_content_title = schema.Bool(
        title=_(u"label_display_content_title", default=u"Display Content Title?"),
        default=False,
        required=True
    )
    
    display_content_description = schema.Bool(
        title=_(u"label_display_content_description", default=u"Display Content Description?"),
        default=True,
        required=True
    )
    
    display_content_image = schema.Bool(
        title=_(u"label_display_content_image", default=u"Display Content Image?"),
        default=True,
        required=True
    )
    
    display_content_text = schema.Bool(
        title=_(u"label_display_content_text", default=u"Display Content Text?"),
        default=True,
        required=True
    )
    
    display_more_link = schema.Bool(
        title=_(u"label_display_more_link", default=u"Display \"More...\" link?"),
        default=True,
        required=True
    )
    
    navigator = schema.Bool(
        title=_(u"label_navigator", default=u"Use Navigator?"),
        description=_(u"description_navigator",
            default=u"Should this Flow View display navigation tabs?"
        ),
        default=True,
        required=True
    )
    
    tab_content = schema.Choice(
        title=_(u"label_tab_content", default=u"Tab Content"),
        description=_(u"description_tab_content",
            default=u"Choose what you want to be displayed in tabs. "
        ),
        default='title',
        vocabulary=SimpleVocabulary([
            SimpleTerm('title', 'title', _(u"label_title", default=u"title")),
            SimpleTerm('description', 'description', _(u"label_description", default=u"description")),
            SimpleTerm('image', 'image', _(u"label_image", default=u"image")),
            SimpleTerm('slidetabs', 'slidetabs', _(u"label_slidetabs", default=u"slidetabs")),
        ]),
        required=True
    )
    
    tab_position = schema.Choice(
        title=_(u"label_tab_position", default=u"Tab Position"),
        description=_(u"description_tab_content",
            default=u"Choose if you want to display tab on the top or the bottom of the content."
        ),
        default='top',
        vocabulary=SimpleVocabulary([
            SimpleTerm('top', 'top', _(u"label_top", default=u"top")),
            SimpleTerm('bottom', 'bottom', _(u"label_bottom", default=u"bottom")),
        ]),
        required=True
    )
    
    use_backnext = schema.Bool(
        title=_(u"label_use_backnext", default=u"Display Back and Next buttons?"),
        default=False,
        required=True
    )

    batch_size = schema.Int(
        title=_(u"label_batch_size", default=u"Batch Size"),
        description=_(u"description_batch_size",
            default=u"Sets the number of items to display per pane. "
        ),
        default=1,
        required=True
    )

    randomize = schema.Bool(
        title=_(u"label_randomize", default=u"Randomize content order"),
        default=False
    )



from Solgema.FlowView.browser.views.flowview import FlowView
from plone.app.contenttypes.browser.folder import FolderView
        
class FlowViewDX(FlowView, FolderView):

    def __init__(self, context, request):
        super(FlowViewDX, self).__init__(context, request)




import logging
from zope.component import getUtilitiesFor
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary
from Solgema.FlowView.interfaces import IFlowViewSettings
from plone.app.vocabularies.catalog import SearchableTextSourceBinder
from plone.app.vocabularies.catalog import SearchableTextSource
from plone.app.vocabularies.catalog import parse_query
from plone.app.customerize import registration
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.component import getGlobalSiteManager
from zope.component import getSiteManager
from zope.interface import implementer
from zope.publisher.interfaces.browser import IBrowserView, IBrowserPage
from Solgema.FlowView.interfaces import IFlowViewMarker
from Solgema.FlowView.interfaces import IFlowViewDisplayType
from zope.schema.interfaces import IVocabularyFactory
from Products.CMFCore.utils import getToolByName
from Solgema.FlowView.config import _
LOG = logging.getLogger(__name__)

class PTGVocabulary(SimpleVocabulary):

    def __init__(self, terms, *interfaces, **config):
        super(PTGVocabulary, self).__init__(terms, *interfaces)
        if 'default' in config:
            self.default = config['default']
        else:
            self.default = None

    def getTerm(self, value):
        """See zope.schema.interfaces.IBaseVocabulary"""
        try:
            return self.by_value[value]
        except KeyError:
            return self.by_value[self.default]
        except:
            raise LookupError(value)

def FlowViewDisplayTypeVocabulary(context):
    terms = []
    utils = list(getUtilitiesFor(IFlowViewDisplayType))
    for name, utility in sorted(utils, key=lambda x: x[1].name):
        if utility.name:
            name = utility.name or name
            terms.append(SimpleTerm(name, name, utility.description))

    return PTGVocabulary(terms,
                default=IFlowViewSettings['display_type'].default)

class FlowViewSearchableTextSource(SearchableTextSource):

    def search(self, query_string):
        results = super(FlowViewSearchableTextSource, self).search(query_string)
        utils = getToolByName(self.context, 'plone_utils')
        query = self.base_query.copy()
        if query_string == '':
            if self.default_query is not None:
                query.update(parse_query(self.default_query, self.portal_path))
        else:
            query.update(parse_query(query_string, self.portal_path))
        try:
            results = self.catalog(**query)
        except:
            results = []

        utils = getToolByName(self.context, 'plone_utils')
        for result in results:
            try:
                if utils.browserDefault(result.getObject())[1][0] ==\
                                                        "flowview":
                    yield result.getPath()[len(self.portal_path):]
            except:
                pass


class FlowViewSearchabelTextSourceBinder(SearchableTextSourceBinder):

    def __init__(self):
        self.query = {'object_provides': IFlowViewMarker.__identifier__}
        self.default_query = 'path:'

    def __call__(self, context):
        return FlowViewSearchableTextSource(
            context,
            base_query=self.query.copy(),
            default_query=self.default_query
        )

@implementer(IVocabularyFactory)
def FlowViewsVocabulary( context ):
    base = [SimpleTerm('content', 'content', _(u"label_content", default=u"Content")),
            SimpleTerm('summary', 'summary', _(u"label_summary", default=u"Summary")),
            SimpleTerm('banner', 'banner', _(u"label_banner", default=u"Banner")),
            SimpleTerm('custom', 'custom', _(u"label_custom", default=u"Custom"))]
    views = registration.getViews(IBrowserRequest)
    for name in [a.name for a in views if IFlowViewMarker in a.required and a.name not in ['flowview-settings', 'flowview-content']]:
        base.append(SimpleTerm(name,name,_(name)))
    return SimpleVocabulary(base)



from setuptools import setup, find_packages
import os

version = '1.0.1'

long_description = '\n\n'.join([
    open('README.rst').read(),
    open('CHANGES.rst').read(),
])

setup(name='Solgema.FlowView',
      version=version,
      description="Solgema",
      long_description=long_description,
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='Solgema, JqueryTools, Flow, Tabs',
      author='Martronic SA',
      author_email='martronic@martronic.ch',
      url='http://www.martronic.ch/Solgema/plone_products/solgema_flowview',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Solgema'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'plone.app.jquerytools'
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.memoize.instance import memoize
from Solgema.FlowView.utils import getDisplayAdapter
from Products.CMFCore.utils import getToolByName
from zope.contentprovider.interfaces import IContentProvider
from Solgema.FlowView.interfaces import IFlowViewSettings, IFlowViewMarker
from zope.component import getMultiAdapter, queryMultiAdapter
from zope.contentprovider.interfaces import ITALNamespaceData
from zope.contentprovider.provider import ContentProviderBase
from zope.interface import implements, alsoProvides, directlyProvides, Interface
from zope.publisher.interfaces.browser import IBrowserRequest
from zope import schema
from zope.component.hooks import getSite
from Products.Five.utilities import marker
try:
    from collective.plonetruegallery.utils import getGalleryAdapter
    hasTrueGallery = True
except:
    hasTrueGallery = False


def jsbool(val):
    return str(val).lower()

class BaseFlowView(BrowserView):

    def __init__(self, context, request):
        super(BaseFlowView, self).__init__(context, request)
        marker.mark( self.context, IFlowViewMarker )
        
    @property
    def macros(self):
        return self.template.macros

class FlowView(BrowserView):

    name = None
    description = None
    schema = None
    userWarning = None
    staticFilesRelative = '++resource++solgemaflowview.resources'
    contentclass = 'page'

    def __init__(self, context, request):
        super(FlowView, self).__init__(context, request)
        self.settings = IFlowViewSettings(context)
        portal_state = getMultiAdapter((context, request),
                                        name='plone_portal_state')
        self.portal_url = portal_state.portal_url()
        self.staticFiles = "%s/%s" % (self.portal_url,
                                      self.staticFilesRelative)
        if hasTrueGallery:
            self.adapter = getGalleryAdapter(self.context, self.request)

    def css(self):
        return ""

    def contentStyle(self):
        out = []
        if getattr(self.settings, 'height', None):
            out.append('height:'+str(self.settings.height)+'px')
        return '; '.join(out)

    def mainClass(self):
        backnextClass = self.settings.use_backnext and ' backnext_enabled' or ''
        borderClass = getattr(self.settings, 'showBorder', True) and ' showBorder' or ''
        return 'useFlowtabs '+self.settings.effect+' '+self.settings.tab_content+backnextClass+borderClass
    
    def padding(self):
        pad = getattr(self.settings, 'flowPadding', 15)
        if pad:
            return str(pad)+'px'
        return '0'

    @memoize
    def get_start_image_index(self):
        if hasTrueGallery and 'start_image' in self.request:
            si = self.request.get('start_image', '')
            images = self.adapter.cooked_images
            for index in range(0, len(images)):
                if si == images[index]['title']:
                    return index
        return 0

    start_image_index = property(get_start_image_index)

    @memoize
    def number_of_items(self):
        if IATTopic.providedBy(self.context):
            return len(self.context.queryCatalog())
        return len(self.context.getFolderContents())
        
    @property
    def showTitle(self):
        return getattr(self.settings, 'showTitle', True)
        
    @property
    def showDescription(self):
        return getattr(self.settings, 'showDescription', True)

    def javascript(self):
        txt = ["""<script type="text/javascript">""",]
        if getattr(self.settings, 'effect', None) == 'custom' and getattr(self.settings, 'custom_effect', None):
            txt.append(self.settings.custom_effect)
        txt.append(self.resizePages())
        txt.append(self.initTabs())
        txt.append(self.activateFlowView())
        if getattr(self.settings, 'invocation_code', None):
            txt.append(self.settings.invocation_code)
        else:
            containerid = 'flow_'+self.context.getId()
            txt.append('activateFlowView($("#'+containerid+'"));')
        txt.append("""</script>""")
        return '\n'.join(txt) 

    def randomizePanes(self):
        if getattr(self.settings, 'randomize', False):
            return """
    var navitop = container.find(".navi.top li");
    if ($(navitop).length > 0) {
        var navitopParent = navitop.parent('ul');
        navitop.detach();
    }
    var navibottom = container.find(".navi.bottom li");
    if ($(navibottom).length > 0) {
        var navitopParent = navitop.parent('ul');
        navitop.detach();
    }
    var panes = container.find(".%(contentclass)s");
    if ($(panes).length > 0) panes.detach();
    var panesOrder = new Array();
    for (i=0;i<$(panes).length;i++) {
        panesOrder.push(i);
    }
    var n = panesOrder.length;
    var tempArr = [];
    for ( var i=0; i<n-1; i++ ) {
        tempArr.push(panesOrder.splice(Math.floor(Math.random()*panesOrder.length),1)[0]);
    }
    tempArr.push(panesOrder[0]);
    panesOrder=tempArr;
    for (i=0;i<n;i++) {
        s = panesOrder[i];
        if ($(navitop).length > 0) {
            $(navitop[s]).appendTo($(navitopParent));
        }
        if ($(navibottom).length > 0) {
            $(navibottom[s]).appendTo($(navibottomParent));
        }
        if ($(panes).length > 0) {
            $(panes[s]).appendTo(container.find(".items"));
        }
    }
""" % {'contentclass':self.context.getId()+'-'+self.contentclass}
        return ''

    def activateFlowView(self):
        return """
function activateFlowView(container) {
    if (!container | typeof(container) == 'function') {
        var container = $("#%(containerid)s");
    }
    if (container.length == 0) return false;
    if (container.find("img").length > 0) {
        container.find("img:last").one("load", function() {
            runFlowView(container);
        }).each(function() {
            if(this.complete) $(this).load();
        }).on("error", function(){
            runFlowView(container);
        });
    } else {
        runFlowView(container);
    }
}
"""% {
    'containerid':'flow_'+self.context.getId(),
    }

    def resizePages(self):
        return """
var pageContentHeight = %(height)s;
function resizePages(container) {
    if ($(window).height()<500) {
        $('#solgemabandeau .page').css('height','300px');
        $('#solgemabandeau .pageContent').css('height','300px');
    } else if (pageContentHeight != null) {
        $('#solgemabandeau .page').css('height', pageContentHeight+'px');
        $('#solgemabandeau .pageContent').css('height', pageContentHeight+'px');
    }
    var panes = container.find(".page");
    var contentWidth = container.find("#flowpanes_container #flowpanes").width();
    var numitems = $(panes).first().find('.pageContent').length;
    var batch_size = %(batch_size)i;
    if ($( document ).width()<768) {
      var batch_size = 1;
    }
    if (batch_size > 1) {
        var rapport = (batch_size-1)/batch_size;
    } else {
        var rapport = 1;
    }
    var rapport = 1;
    var page_width = contentWidth/batch_size;
    var content_width = page_width/numitems;
    $(panes).each( function(index) {
        var page_marginWidth = $(this).outerWidth(true)-$(this).width();
        var pagewidth = page_width-(page_marginWidth*rapport);
        $(this).width(pagewidth);
        $(this).children().each( function(index) {
            var content_marginWidth = $(this).outerWidth(true)-$(this).width();
            $(this).width( pagewidth-content_marginWidth );
        });
    });
    var flowPanesHeight = 0;
    $(panes).each( function(index) {
        if( $(this).outerHeight(true) > flowPanesHeight) flowPanesHeight = $(this).outerHeight(true);
        if($(this).children().length == 1) {
            $(this).children().each( function(index, child) {
                if( $(child).outerHeight(true) > flowPanesHeight) flowPanesHeight = $(child).outerHeight(true);
            });
        }
    });
    $(panes).each( function(index) {
        var content_marginHeight = $(this).outerHeight(true)-$(this).height();
        $(this).css("height", flowPanesHeight-content_marginHeight);
        if($(this).children().length == 1) {
            var content_marginHeight = $(this).outerHeight(true)-$(this).height();
            $(this).css("height", flowPanesHeight-content_marginHeight);
        }
    });
    container.find("#flowpanes").height(flowPanesHeight);
    var tooldata = container.find("#flowpanes").data("%(tooldata)s");
    if (tooldata) tooldata.next();
    if (pageContentHeight == null) {
        pageContentHeight = flowPanesHeight;
    }
};
""" % {'height':getattr(self.settings, 'height', None) and str(self.settings.height) or 'null',
       'batch_size':self.settings.effect in ['swing', 'linear'] and getattr(self.settings, 'batch_size', 1) or 1,
       'tooldata':self.settings.effect in ['default', 'fade', 'ajax', 'slide', 'custom'] and 'tabs' or 'scrollable'
    }

    def initTabs(self):
        return """
var panes_number = 0;
function runFlowView(container) {
    %(randomize)s
    var panes = container.find(".page");
    panes_number = $(panes).length;
    if (panes_number == 0) return false;
    containerid = container.attr('id');
    var itemid = containerid.replace('flow_','');
    var paneid = itemid+'-pane';
    container.find(".documentActions").detach().insertBefore($("#portal-column-content #region-content:first"));
    $(panes).each(function (index) {
            %(js_remove_title)s
            %(js_remove_description)s
            $(this).find("#review-history").remove();
            var lastP = $(this).find("p").last();
            if ($(lastP).html() != null) {
                if ($(lastP).html().trim() == '&nbsp;') {
                    $(lastP).remove();
                }
            }
    });
    container.css("display","block").addClass("flowEnabled");
    container.parent().addClass("flowEnabled");
    var batch_size = %(batch_size)i;
    if ($( document ).width()<768) {
      var batch_size = 1;
    }
    if (batch_size > 1) {
        container.addClass("batched");
        container.addClass("manyItems");
    }
    $(panes).each( function(index) {
        $(this).attr("id", paneid+index);
    });
    resizePages(container);
    $(window).resize(function () {
        //solgema.js needed!
        waitForFinalEvent(function(){
            resizePages(container);
            }, 100, "resizePages"+containerid);
    });
    for (i=1;i<%(batch_size)i;i++) {
        var cloned = container.find("#"+itemid+"-pane"+i).clone().addClass('cloned').appendTo("#flow_"+itemid+" #flowpanes .items:first");
    }
    %(effectjs)s
""" % {
    'randomize'            :self.randomizePanes(),
    'effect'               :self.settings.effect,
    'effectjs'             :self.settings.effect in ['default', 'fade', 'ajax', 'slide', 'custom'] and self.standardJavascript() or self.scrollableJavascript(),
    'batch_size'           :self.settings.effect in ['swing', 'linear'] and getattr(self.settings, 'batch_size', 1) or 1,
    'js_remove_title'      :not self.settings.display_content_title and """$(this).find("h1.documentFirstHeading:first, h2.tileHeadline:first").first().remove();""" or '',
    'js_remove_description':not self.settings.display_content_description and """$(this).find("p.description:first, p.documentDescription:first").first().remove();""" or ''
    }

    def scrollableJavascript(self):
        return """$("#"+containerid+" #flowtabs ul li a:first").addClass("current");
    container.find("#flowpanes").scrollable({
        circular   : true,
        easing     : "%(effect)s",
        speed      : %(speed)i,
        vertical   : %(vertical)s,
        next       : "#"+containerid+" .forward",
        prev       : "#"+containerid+" .backward",
//        mousewheel : true,
        activeClass: "current"
    })%(activateAutoscrollTimed)s.navigator({
        navi       : "#"+containerid+" .navi ul",
        naviItem   : "a",
        activeClass: "current"
    });
};""" % {
    'effect'                 :self.settings.effect,
    'speed'                  :self.settings.speed,
    'vertical'               :jsbool(self.settings.vertical),
    'activateAutoscrollTimed':self.activateAutoscrollTimed()
    }

    def activateAutoscrollTimed(self):
        if not self.settings.timed:
            return ''
        return """.autoscroll({
    interval: %(interval)s,
    autoplay: %(autoplay)s,
    autopause: %(autopause)s,
    })""" % {
    'interval' :str(self.settings.interval),
    'autoplay' :jsbool(self.settings.autoplay),
    'autopause':jsbool(self.settings.autopause),
    }

    def standardJavascript(self):
        return """
    $("#"+containerid+" .navi ul").tabs("#"+containerid+" .items:first > div",IBrowserRequest
        {
        rotate: true,
        effect: "%(effect)s",
        fadeInSpeed: %(fadeInSpeed)s,
        fadeOutSpeed: %(fadeOutSpeed)s,
    })%(activateTimed)s;
};""" % {
    'effect'               :self.settings.effect,
    'speed'                :self.settings.speed,
    'vertical'             :jsbool(self.settings.vertical),
    'fadeInSpeed'          :str(self.settings.fadeInSpeed),
    'fadeOutSpeed'         :str(self.settings.fadeOutSpeed),
    'activateTimed'        :self.activateTimed()
    }

    def activateTimed(self):
        if not self.settings.timed and not self.settings.use_backnext:
            return ''
        txt = '.slideshow({'
        if self.settings.timed:
            txt += """interval: "%(interval)i",
    next     : "#"+containerid+" .forward",
    prev     : "#"+containerid+" .backward",
    autoplay : %(autoplay)s,
    autopause: %(autopause)s,""" % {
    'interval'   :self.settings.interval,
    'autoplay'   :jsbool(self.settings.autoplay),
    'autopause'  :jsbool(self.settings.autopause),
    }
        return txt+'})'
    
    def __call__(self):
        if self.settings.content_layout not in ['content', 'summary', 'banner', 'custom']:
            view = queryMultiAdapter((self.context, self.request), name=self.settings.content_layout, default=None)
            if view:
                return view()
        return self.index()

class IBannerView(Interface):
    item = schema.Field(title=u"Item")
    settings = schema.Field(title=u"Settings")

directlyProvides(IBannerView, ITALNamespaceData)
   
class BannerView(ContentProviderBase):
    implements(IBannerView)
    
    item = None
    settings = None

    index = ViewPageTemplateFile('banner-view.pt')

    def render(self):
        return self.index()
        
#    def __init__(self, context, request, view):
#        super(BannerView, self).__init__(context, request, view)

    def update(self):
        super(BannerView, self).update()
#        self.settings = IFlowViewSettings(self.__parent__.context)
        portal_state = getMultiAdapter((self.context, self.request),
                                        name='plone_portal_state')
        self.portal = portal_state.portal()
        self.image = self.getImage()
        self.imageDict = self.getImageDict()
        if self.item == None:
            self.item = self.context

    @memoize
    def getImages(self):
        results = []
        if hasattr(self.item, 'getObject'):
            catalog = getToolByName(self.item, 'portal_catalog')
            if isinstance(getattr(self.item, 'image_assoc', None), list) and len(getattr(self.item, 'image_assoc', [])) > 0:
                results = catalog.searchResults({'UID':self.item.image_assoc[0], 'Language':'all'})
            elif self.item.usecontentimage:
                path = self.item.getPath()
                if path:
                    results = catalog.searchResults(path=path, portal_type='Image', sort_on='getObjPositionInParent')
        else:
            if getattr(self.item, 'displayimginsummary', False):
                try:
                    images = self.item.getField('image_assoc').get(self.item)
                except:
                    pass
                if images:
                    catalog = getToolByName(self.item, 'portal_catalog')
                    results = catalog.searchResults({'UID':images[0].UID(), 'Language':'all'})
        return results
        
    def getImage(self):
        images = self.getImages()
        if images:
            return images[0].getObject()
        return []

    def imageWidth(self):
        if self.image:
            return self.image.getField('image').getSize(self.image)[0]
        return None

    def imageHeight(self):
        if self.image:
            return self.image.getField('image').getSize(self.image)[1]
        return None

    def getImageDict(self):
        if not self.image:
            return None
        return {'image_url'   :self.image.absolute_url(),
                'image_width' :self.imageWidth(),
                'image_height':self.imageHeight(),
                'image_title' :self.image.Title()
                }

    def getStyle(self):
        out = []
        if self.imageDict:
            out.append('background-image:url('+self.imageDict['image_url']+')')
        return '; '.join(out)
